#include "charbuffer.hpp"
#include "pixelbuffer.hpp"
#include <string.h>
#define max(a, b)		(	((a) >= (b))	? (a) : (b)	)
#define min(a, b)		(	((a) <= (b))	? (a) : (b)	)
//void debugLog(int line, const char *format, ...);

CharBuffer::CharBuffer(int width,int height,int vis_width,int vis_height)
: _width(width),
  _height(height),
  _cells(new CellBuffer [width*height]),
  _active_fg_color(2),		//	Green
  _active_bg_color(0),		//	Black
  _cursor(vis_width,vis_height),
  _font_ptr(new FontInfo)
{
  int row = 0;
  for (;row!=height;++row) {
    clearRow(row);
  }
}

bool CharBuffer::resize(int new_visible_width, int new_height)
	{
        assert(new_visible_width>=0);
        assert(new_height>=0);
        if (new_visible_width==visibleWidth() && new_height==height()) {
          // it's ok, we have no need to resize
          return false;
        }
        int width = max(new_visible_width, _width);
           // use a large enough buffer for both orientations
        int height = new_height;
	int size = width * height;
	CellBuffer *tmp = new CellBuffer [size];
	int row;
	int maxCol = min(width, _width);
	int maxRow = min(height, _height);


	//memset(tmp, ' ', size);		//	fill all rows/cols with spaces

	if(new_visible_width > _width)
		{
		//	we are expanding the width, so we must pad out all the cols
		CellBuffer *o = tmp;
		for(row = 0; row < maxRow; row++, o += new_visible_width)
			{
			for (int col = _width; col < new_visible_width;++col) {
				o[col].ch = ' ';
				o[col].fg_color = _active_fg_color;
				o[col].bg_color = _active_bg_color;
				o[col].flags = _active_flags;
				}
			}
		}
	if(new_height > _height)
		{
		CellBuffer *o = tmp + _height;
		for(row = maxRow; row < new_height; row++, o += new_visible_width)
			{
			for (int col = 0; col < new_visible_width;++col) {
				o[col].ch = ' ';
				o[col].fg_color = _active_fg_color;
				o[col].bg_color = _active_bg_color;
				o[col].flags = _active_flags;
				}
			}
		}


	for(row = 0; row < maxRow; row++)
		{
		memcpy(tmp + ((height-1-row) * width), _cells + ((_height-1-row) * _width), maxCol * sizeof(CellBuffer));		//	copy all the data
		}
	//	clean up old data
	delete [] _cells;

	//	store the new values here
	_cells = tmp;
	_width = width;					//	store the new width
	_height = height;

	_cursor.setScreenWidth(new_visible_width);
	return true;
	}

int CharBuffer::resizeToPixels(int width, int height)
	{
        FontInfo &font = this->font();
	return resize(width / font.char_width(), height / font.char_height());
	}

int CharBuffer::addLines(int lines)
	{
	return resize(_width, _height + lines);
	}

void CharBuffer::clearRow(int row)
{
  assert(row<height());
  int width = _width;
  int col = 0;
  for (;col!=width;++col) {
    CellBuffer &theCell = cell(row,col);
    theCell .ch = ' ';
    theCell .fg_color = _active_fg_color;
    theCell .bg_color = _active_bg_color;
    theCell .flags = _active_flags;
  }
}

void CharBuffer::scrollUp(int count)
{
  int width = _width;
  int height = _height;
  int row = 0;
  for (;row<height-count;++row) {
    int col = 0;
    for (;col!=width;++col) {
      cell(row,col) = cell(row+count,col);
    }
  }
  for (;row<height;++row) {
    clearRow(row);
  }
}

void CharBuffer::scrollDown(int count)
{
  int width = _width;
  int height = _height;
  int row = height-1;
  for (;row-count>=0;--row) {
    int col = 0;
    for (;col!=width;++col) {
      cell(row,col) = cell(row-count,col);
    }
  }
  for (;row>=0;--row) {
    clearRow(row);
  }
}
