#include "keyman.hpp"


static int TranslateRed(int key_code)
{
               // @ABCDEFGHIJKLMN OPQRSTUVWXYZ
  char table[] = "0&#84156$%!:';?\"=/2-3)9+7(*";
  if (key_code>=64 && key_code<='Z') {
    return table[key_code-64];
  }
  return 0;
}


void 
  KeyManager::keyPressed(
    int key_code,
    bool shift_pressed,
    bool alt_pressed,
    bool ctrl_pressed,
    bool meta_pressed
  )
{
  if (key_code==231) {
    // gesture area tap
    gesture_hold_state = (gesture_hold_state+1)%3;
  }
  else if (key_code==129) {
    // orange/red key
    red_state = (red_state+1)%3;
  }
  else if (key_code==16 || (key_code==0 && shift_pressed)) {
    // shift key
    shift_state = (shift_state+1)%3;
  }
  else if (key_code==17 || key_code==131) {
    // sym key or ctrl key
    sym_state = (sym_state+1)%3;
  }
  else {
    if (key_code>='a' && key_code<='z') {
      key_code -= 32;
    }
    if (key_code>='A' && key_code<='Z') {
      if (red_state || alt_pressed) {
        key_code = TranslateRed(key_code);
      }
      else if (gesture_hold_state || meta_pressed) {
        switch (key_code) {
          case 'E': sendHome(); break;
          case 'R': sendUp(); break;
          case 'T': sendPrior(); break;
          case 'D': sendLeft(); break;
          case 'F': break;
          case 'G': sendRight(); break;
          case 'X': sendEnd(); break;
          case 'C': sendDown(); break;
          case 'V': sendNext(); break;
          case '0': sendInsert(); break;
          case 190: sendDelete(); break;
        }
        key_code = 0;
      }
      else if (sym_state || ctrl_pressed) {
        // control characters
        key_code = key_code-64;
      }
      else if (!shift_state && !shift_pressed) {
        key_code += 32;
      }
    }
    else if (key_code=='0' && !red_state) {
      key_code = '@';
    }
    else if (key_code==32 && red_state) {
      // escape
      key_code = 27;
    }
    else if (key_code==188) {
      if (red_state) {
        key_code = '_';
      }
      else {
        key_code = ',';
      }
    }
    else if (key_code==190) {
      if (red_state) {
        key_code = '|';
      }
      else  {
        key_code = '.';
      }
    }
    if (key_code!=0 && key_code<128) {
      sendChar(key_code);
    }
    if (red_state==1) {
      red_state = 0;
    }
    if (shift_state==1) {
      shift_state = 0;
    }
    if (sym_state==1) {
      sym_state = 0;
    }
    if (gesture_hold_state==1) {
      gesture_hold_state = 0;
    }
  }
  flush();
}


void KeyManager::sendF(int num)
{ 
  assert(num>=1 && num<=12);
  sendCSI(); 
  if (num<=5) {
    sendChar('1');
    sendChar('0'+num);
  }
  else if (num<=8) {
    sendChar('1');
    sendChar('0'+num+1);
  }
  else {
    sendChar('2');
    sendChar('0'+num-9);
  }
  sendChar('~');
}
